;***************************************************************************
; 
; File Name		:'Main.asm"
; Title			:TriTone DeModulator 
; Date			:2002.04.17.
; Version		:1.2.0
; Support telephone	:+36-70-333-4034
; Support fax		:
; Support Email		:info@vfx.hu
; Target MCU		:AT90S8515
; 
;***************************************************************************
;	D E S C R I P T I O N
;
;  ANA0,ANA1: Analog comparator input
;  TxD:   RS232 Output 
;
;***************************************************************************
;	M O D I F I C A T I O N   H I S T O R Y 
;
;
;       rev.      date      who  	why
;	----	----------  ---		------------------------------------
;	0.01	2002.04.17  VFX		TriTone DeModulator, Creation
;	1.20	2002.05.30. VFX,Laca	fixed jitter, final ver.
;
;
;***************************************************************************
;Hardware
;***************************************************************************
;*
;*	Kvarc: f=4.000 MHz (T=125.00ns)
;*
;***************************************************************************
;
;
;
.include "8515def.inc"


;***************************************************************************
;* Const Def



;* periodusidok @4MHz: (meres felmeno eltol felmeno elig, max 10%)
;* 1200Hz  -> 3334+-333 -> Low bit
;* 1500Hz  -> 2666+-266 -> High bit
;*  960Hz  -> 4166+-416 -> Guard (Tone)


.EQU	LowFrekv =  3334
.EQU	LowDelatF =333

.EQU	HiFrekv = 2666
.EQU	HiDelatF =266
		
.EQU	GuardFrekv = 4166
.EQU	GuardDeltaF = 416


.EQU	TimeOut = 5300		;nincs guard jel



;* UBR=INT(clk/(16*Baud)-1+0.5)

.EQU	BaudRate = 207		;25 -: 9600 Baud @4.000MHz
				;207 -:1200 Baud @4.000MHz

;**************************************************************************
;* Hardware Def.

.equ	LEDPORT = PORTD
.equ	LEDDIR = DDRD
.equ	LED = 4



;***************************************************************************
;**** VARIABLES
.DSEG


;***************************************************************************
.ESEG
		.DB 	"TriTone DeModulator [2002.04.17.] "
		.DB	"Latest Upd.: 2002.05.30."
		.DB	"Rev:1.2.0 "


;*****************************************************************************
;****		I N T E R R U P T S
;****
;*****************************************************************************
.CSEG
	rjmp RESET		;Reset Handle
	rjmp EXT_INT0		;IRQ0 Handle
	rjmp EXT_INT1		;IRQ1 Handle
	rjmp TIM1_CAPT		;Timer1 capture Handle
	rjmp TIM1_COMPA		;Timer1 compare Handle
	rjmp TIM1_COMPB		;Timer1 compare Handle
	rjmp TIM1_OVF		;Timer1 overflow Handle
	rjmp TIM0_OVF		;Timer0 overflow Handle
	rjmp SPI_STC		;SPI Transfer Complete Handle
	rjmp UART_RXC		;UART RX Complete Handle
	rjmp UART_DRE		;UDR Empty Handle
	rjmp UART_TXC		;UART TX Complete Handle
	rjmp ACI_INT		;Analog Comparator Handle


;********************************
;* RESET 
;********************************
RESET:		CLI			;GLOBAL INTERRUP DIS.	

		ldi r16, high(RAMEND)	;setup stack
		out SPH,r16
		ldi r16, low(RAMEND)
		out SPL,r16

		LDI	R16,0b00101010	;Sleep enable, Idle mode,INT0, INT1 edge sens.
		OUT	MCUCR,R16	;7. = 1 SRE External SRAM Enabled
					;6. = 1 SRW Ext. SRAM Wait State Enabled
					;5. = 1 Sleep Enable
					;4. = 0 SLEEP => Idle Mode
					;     1 SLEEP => Power Down Mode
					;3:2 = Inerrupt Sense Control for INT1
					; 00 -> Low Level INT1
					; 01 -> Reserved
					; 10 -> Falling Edge INT1
					; 11 -> Rising Edge INT1
					;1:0 = Inerrupt Sense Control for INT0
					; 00 -> Low Level INT0
					; 01 -> Reserved
					; 10 -> Falling Edge INT0
					; 11 -> Rising Edge INT0



		LDI	R16,0b00000000	;INT0, INT1 Disabled
		OUT	GIMSK,R16	;7. = 1 EXT INT1 ACTIVE
					;6. = 1 EXT INT0 ACTIVE


;***************************************
;**  Timer0 Init		   *****
;**
InitTimer0:	LDI	R16,0b00000000
		OUT	TCCR0,R16	;TC0 Control Register
					;2:0 Clock Select
					; 000 -> Stop
					; 001 -> CK
					; 010 -> CK/8
					; 011 -> CK/64
					; 100 -> CK/256
					; 101 -> CK/1024
					; 110 -> External pin T0 falling edge
					; 111 -> External pin T0 risinrg edge


;UART Init 
		LDI	R16,0b00011000		;RXCIE,TXCIE,UDRIE,RXEN,TXEN,CHR9,RXB8,TXB8
		OUT	UCR,R16
		LDI	R16,BaudRate
		OUT	UBRR,R16		;BaudRate Gen.
		ldi	R16,255
		out	USR,R16			;status torlese	

;Analog Comparator Init
		ldi	R16,0b00010111
					;7 - ACD: Analog Comparator Disable
					;5 - ACO: Analog Comparator Output
					;4 - ACI: Analog Comparator Interrupt Flag
					;3 - ACIE: Analog Comparator Interrupt Enable
					;2 - ACIC: Analog Comparator Input Capture0 Enable
					;[1:0] - ACIS1, ACIS0: Analog Comparator Interrupt Mode Select
					; 00 Comparator Interrupt on Output Toggle                              
					; 01 Reserved
					; 10 Comparator Interrupt on Falling Output Edge
					; 11 Comparator Interrupt on Rising Output Edge
		out	ACSR,R16
		sbi	ACSR,ACI	;clear pending AComp interrupt


		cbi	PORTB,PB2
		cbi	PORTB,PB3
		cbi	DDRB,PB2
		cbi	DDRB,PB3	;Copmarator pin tri-state


;Timer1 init
		LDI	R16,0b01001000
		OUT	TIMSK,R16 	;7. = 1 TC1 OWRF EN.  *
					;6. = 1 TC1 COMPA MATCH EN.
					;5. = 1 TC1 COMPB MATCH EN.
					;3. = 1 TC1 INP CAPTURE EN.
					;1. = 1 TC0 OWRF EN.   *

		LDI	R16,0b00000000
		OUT	TCCR1A,R16	;Non-invert 8bit PWM, 
					;7:6 00 => OC1 not connected
					;    01 => OC1 not connected
					;    10 => OC1 non-inverted PWM
					;    11 => OC1 inverted PWM
					;1:0 00 => PWM Disabled
					;    01 =>  8 bit PWM
					;    10 =>  9 bit PWM
					;    11 => 10 bit PWM


		LDI	R16,0b01001001
		OUT	TCCR1B,R16	;7. = 1 Input Capture Noise Canceller En.
					;6. = 1 Input Capture ICP Rising edge
					;3. = 1 Reset TC1 when copare match
					;2:0 Clock select
					;   000 => TC1 Stop
					;   001 => Ck
					;   010 => Ck/8
					;   011 => Ck/64
					;   100 => Ck/256
					;   101 => Ck/1024
					;   110 => External T1 Falling Edge
					;   111 => External T1 rising Edge


		ldi	R16,High(TimeOut)
		OUT	OCR1AH,R16
		ldi	R16,Low(TimeOut)
		OUT	OCR1AL,R16		;compare A reg = Time Out, nincs elvaltas a radioban
		
		ldi	r16,0
		out	SPCR,R16

		ldi	R16,0b11101010		;Timer1 interruptok torlese
		out	TIFR,R16
	

		cbi	LEDPORT,LED	;debug LED
		sbi	LEDDIR,LED



		SEI			;SET GLOBAL INT EN.
		RJMP	Main

;*******************
EXT_INT0:	RETI
EXT_INT1:	RETI

;*******************

;Analog Comp felfuto el jott

TIM1_CAPT:
		clr	R26
		out	TCNT1H,R26
		out	TCNT1L,R26		;CNT=0 timer lenullazva
		RETI

;Time out, nincs elvaltas a radioban

TIM1_COMPA:	ldi	R25,1
		RETI

TIM1_COMPB:	RETI
TIM1_OVF:	RETI

;********************
;* Timer0 Int
;* 
TIM0_OVF:	RETI
SPI_STC:	RETI

;*****************
;* UART RX COMPLET 
UART_RXC:	RETI

;*******************
UART_DRE:	RETI

;******************
;* UART TX COMPLET 
UART_TXC:	RETI

ACI_INT:	RETI


;***********************************************************************
;***********************************************************************
;********     M     A      I      N				********
;********							********
;***********************************************************************
Main:
		sbi	LEDPORT,LED		;LED nem eg
		clr	r24			;periodus szamlalo
Cikl:		clr	R25
		sleep
		nop

		cpi	R24,2
		brsh	cikl1
		inc	R24

cikl1:
		cpi	R25,1			;Time Out volt?	
		breq	Main


		rcall	GetFrekv
		cpi	R28,0
		breq	Main

		cbi	LEDPORT,LED		;LED eg

		cpi	R28,1			;guard jel volt?
		breq	cikl

		cpi	R24,2			;Ha nincs legalabb 2 guard akkor vissza
		brcs	Main


		clr	R0			;R0-ban gyujtjuk a biteket
		ldi	R27,8			; 8 adatbit kell
		rjmp	LoopIn

GetByte:	clr	R25
		sleep
		nop

		cpi	R25,1			;Time Out volt?	
		breq	Main
		
		rcall	GetFrekv
		cpi	R28,0
		breq	Main
		cpi	R28,1
		breq	Cikl

LoopIn:		cpi	R28,3
		breq	OneBit
		clc
		ror	R0
		rjmp	GetByte1

OneBit:  	sec
		ror	R0

GetByte1:	dec	R27
		brne	GetByte

SendByte:
		out	UDR,R0
		rjmp	Cikl


;*******************************************************
;**

GetFrekv:
		in	ZL,ICR1L
		in	ZH,ICR1H			;Input capture register erteke

		ldi	R16,low(GuardFrekv-GuardDeltaF)
		ldi	R17,high(GuardFrekv-GuardDeltaF)
		cp	ZL,R16
		cpc	ZH,R17
		brcs	guardrosz
		ldi	R16,low(GuardFrekv+GuardDeltaF)
		ldi	R17,high(GuardFrekv+GuardDeltaF)
		cp	ZL,R16
		cpc	ZH,R17
		brcc	guardrosz

		ldi	R28,1				;Guard Jel volt
		ret

guardrosz:
		ldi	R16,low(LowFrekv-LowDelatF)
		ldi	R17,high(LowFrekv-LowDelatF)
		cp	ZL,R16
		cpc	ZH,R17
		brcs	lowrosz
		ldi	R16,low(LowFrekv+LowDelatF)
		ldi	R17,high(LowFrekv+LowDelatF)
		cp	ZL,R16
		cpc	ZH,R17
		brcc	lowrosz

		ldi	R28,2
		ret

lowrosz:
		ldi	R16,low(HiFrekv-HiDelatF)
		ldi	R17,high(HiFrekv-HiDelatF)
		cp	ZL,R16
		cpc	ZH,R17
		brcs	hirosz
		ldi	R16,low(HiFrekv+HiDelatF)
		ldi	R17,high(HiFrekv+HiDelatF)
		cp	ZL,R16
		cpc	ZH,R17
		brcc	hirosz

		ldi	R28,3
		ret

hirosz:		ldi	R28,0				;semmire nem jo
		ret



TogleLED:	SBIS	LEDPORT,LED		;kihagy, ha LED=1
		 RJMP	Led_be 
		CBI	LEDPORT,LED		;LED eg
		ret
Led_be:		SBI	LEDPORT,LED		;LED nem eg
		ret



